import subprocess
import re
from datetime import datetime

def get_processes_with_keyword(keyword):
    processes = []
    try:
        # 运行 ps aux 命令并使用 grep 过滤包含特定关键词的行
        result = subprocess.check_output(['ps', 'aux']).decode('utf-8')
        lines = result.split('\n')
        # 遍历每一行，如果包含关键词，则提取进程ID和相关信息
        for line in lines:
            if keyword in line and 'grep' not in line:
                parts = line.split()
                pid = parts[1]
                command = ' '.join(parts[10:])
                # 使用正则表达式提取 linkname 和 user 信息
                linkname_match = re.search(r'linkname\s+(\S+)', command)
                user_match = re.search(r'user\s+(\S+)', command)
                if linkname_match and user_match:
                    linkname = linkname_match.group(1)
                    user = user_match.group(1)
                    # 获取ppp接口信息
                    ppp_interface_info = get_ppp_interface_info(linkname)
                    processes.append({"pid": pid, "linkname": linkname, "user": user, "ppp_interface_info": ppp_interface_info})
    except subprocess.CalledProcessError as e:
        print(f"Error fetching process list: {e}")
    return processes

def get_ppp_interface_info(linkname):
    try:
        # 使用 ip addr 命令获取指定 linkname 对应的 ppp 接口信息
        result = subprocess.check_output(['ip', 'addr', 'show', linkname]).decode('utf-8')
        # 使用正则表达式提取 IP 地址信息
        ip_match = re.search(r'inet\s+(\d+\.\d+\.\d+\.\d+)', result)
        if ip_match:
            ip_address = ip_match.group(1)
            return ip_address
        else:
            return "N/A"
    except subprocess.CalledProcessError as e:
        print(f"Error fetching PPP interface info for linkname {linkname}: {e}")
        return "N/A"

def get_dial_status(pid, log_file):
    status = "N/A"
    try:
        # 使用 grep 命令过滤包含特定进程ID的行
        result = subprocess.check_output(['cat', log_file]).decode('utf-8')
        lines = result.split('\n')
        for line in lines:
            if f"pppd[{pid}]" in line and "authentication" in line:
                # 使用正则表达式提取authentication成功或失败后面的消息作为状态
                status_match = re.search(r'authentication\s+succeeded:\s*(.+)', line)
                if status_match:
                    status = status_match.group(1)
                    break
                else:
                    status_match = re.search(r'authentication\s+failed:\s*(.+)', line)
                    if status_match:
                        status = status_match.group(1)
                        break
    except subprocess.CalledProcessError as e:
        print(f"Error fetching dial status: {e}")
    return status

# 获取当前日期并格式化为字符串
current_date = datetime.now().strftime("%Y%m%d")
log_file = f"/var/log/pppoe-{current_date}.log"  # 构建日志文件路径

# 测试函数
keyword = "pppd"
processes = get_processes_with_keyword(keyword)
for process in processes:
    dial_status = get_dial_status(process["pid"], log_file)
    print("PID:", process["pid"], "| Linkname:", process["linkname"], "| User:", process["user"], "| Dial Status:", dial_status, "| IP Address:", process["ppp_interface_info"])
